@@ -5,11 +5,11 @@
 extern "C" {
 #endif
 
-#define XMP_VERSION "4.1.5"
-#define XMP_VERCODE 0x040105
+#define XMP_VERSION "4.2.0"
+#define XMP_VERCODE 0x040200
 #define XMP_VER_MAJOR 4
-#define XMP_VER_MINOR 1
-#define XMP_VER_RELEASE 5
+#define XMP_VER_MINOR 2
+#define XMP_VER_RELEASE 0
 
 #if defined(_WIN32) && !defined(__CYGWIN__)
 # ifdef BUILDING_DLL
@@ -46,6 +46,9 @@ extern "C" {
 #define XMP_PLAYER_FLAGS	4	/* Player flags */
 #define XMP_PLAYER_CFLAGS	5	/* Player flags for current module */
 #define XMP_PLAYER_SMPCTL	6	/* Sample control flags */
+#define XMP_PLAYER_VOLUME	7	/* Player module volume */
+#define XMP_PLAYER_STATE	8	/* Internal player state */
+#define XMP_PLAYER_SMIX_VOLUME	9	/* SMIX volume */
 
 /* interpolation types */
 #define XMP_INTERP_NEAREST	0	/* Nearest neighbor */
@@ -56,6 +59,11 @@ extern "C" {
 #define XMP_DSP_LOWPASS		(1 << 0) /* Lowpass filter effect */
 #define XMP_DSP_ALL		(XMP_DSP_LOWPASS)
 
+/* player state */
+#define XMP_STATE_UNLOADED	0	/* Context created */
+#define XMP_STATE_LOADED	1	/* Module loaded */
+#define XMP_STATE_PLAYING	2	/* Module playing */
+
 /* player flags */
 #define XMP_FLAGS_VBLANK	(1 << 0) /* Use vblank timing */
 #define XMP_FLAGS_FX9BUG	(1 << 1) /* Emulate FX9 bug */
@@ -84,6 +92,7 @@ extern "C" {
 #define XMP_ERROR_DEPACK	5	/* Error depacking file */
 #define XMP_ERROR_SYSTEM	6	/* System error */
 #define XMP_ERROR_INVALID	7	/* Invalid parameter */
+#define XMP_ERROR_STATE		8	/* Invalid player state */
 
 struct xmp_channel {
 	int pan;			/* Channel pan (0x80 is center) */
@@ -301,6 +310,16 @@ EXPORT int         xmp_channel_vol     (xmp_context, int, int);
 EXPORT int         xmp_set_player      (xmp_context, int, int);
 EXPORT int         xmp_get_player      (xmp_context, int);
 EXPORT int         xmp_set_instrument_path (xmp_context, char *);
+EXPORT int         xmp_load_module_from_memory (xmp_context, void *, long);
+
+/* External sample mixer API */
+EXPORT int         xmp_start_smix       (xmp_context, int, int);
+EXPORT void        xmp_end_smix         (xmp_context);
+EXPORT int         xmp_smix_play_instrument(xmp_context, int, int, int, int);
+EXPORT int         xmp_smix_play_sample (xmp_context, int, int, int, int);
+EXPORT int         xmp_smix_channel_pan (xmp_context, int, int);
+EXPORT int         xmp_smix_load_sample (xmp_context, int, char *);
+EXPORT int         xmp_smix_release_sample (xmp_context, int);
 
 #ifdef __cplusplus
 }
